#! /bin/sh
iot_upgrade=$1

if [ $# -gt 1 ] ; then
  dir=$2                          
else                                         
  dir=/media/mmcblk0p1                       
fi                                           

file=root_uImage
bfile=bImage
kernel_block=851968
burn_cmd=/bin/mtd_write
target_md5=`cat $dir/_md5_`
target_b_md5=`cat $dir/_bmd5_`
boot_block=196608
echo 3 > /proc/sys/vm/drop_caches

upgrade_result()
{
   if [ x$iot_upgrade = x"byiot" ]
   then
       if [ $1 -eq 1 ]
       then
          echo upgrade ok

          ver=`cat $dir/ver`
          echo $ver
          if [ ! x$ver = x ]
          then
            echo "d 83$ver" > /tmp/mqttdownfifo 
          else
            echo "d 83" > /tmp/mqttdownfifo 
          fi
          
       elif [ $1 -eq 2 ]
       then
          echo upgrade fail file not valid
          echo "d 14" > /tmp/mqttdownfifo 
       fi
   fi
}



clear_history()
{
   rm -f $dir/bak-root_uImag	
   rm -f $dir/install.sh.bak
   rm -f $dir/prt_com.log*
   rm -f $dir/iot-linkit.log*
   rm -f $dir/httpclient.log*  
   rm -f update.log
}
post_pro_file()
{
      if [ -f $dir/creality/batch-update ] 
      then
        echo do not rm
      elif [ -f $dir/creality/debug ] 
      then
        mv -f $dir/$file $dir/bak-root_uImag
      else
        rm -f $dir/$file 
        rm -f $dir/$bfile
        rm -f $dir/_*md5_ 
        rm -f $dir/install.sh 
      fi
}


chkbupdate()
{
  target_tick=`date -d "2020-8-14 14:0:0" +%s`
  y=`uname -a | awk '{print $10}'`
  mon=`uname -a | awk '{print $6}'`
  d=`uname -a | awk '{print $7}'`

  case $mon in
  Jan )
     mon=1
     ;;

  Feb )
     mon=2
     ;;

  Mar )
     mon=3
     ;;

  Apr )
     mon=4
     ;;

  May )
     mon=5
     ;;

  Jun )
    mon=6
   ;;
   
  Jul )
     mon=7
     ;;

  Aug )
     mon=8
     ;;

  Sep )
     mon=9
     ;;

  Oct )
     mon=10
     ;;

  Nov )
     mon=11
     ;;

  Dec )
     mon=12
     ;;

   esac

  date="$y-$mon-$d `uname -a | awk '{print $8}'`"
  release_tick=`date -d "$date" +%s`
  if [ $target_tick -gt $release_tick ]
  then 
    target_b_md5=`cat $dir/_bmd5_`
    real_b_md5=`md5sum $dir/$bfile | awk '{print $1}'`
    if [ $target_b_md5 == $real_b_md5 ] 
    then
      return 1
    fi 
  fi
  return 2
}

update_boot()
{
  chkbupdate 
  if [ $? == 1 ]
  then    
    echo $burn_cmd -o 0 -l $boot_block write $dir/$bfile Bootloader
    $burn_cmd -o 0 -l $boot_block write $dir/$bfile Bootloader
  fi
}

#if diff $dir/ver /home/box_version
#then
#  echo no need to upgrade
#  exit 0
#fi

cal_size()
{
  size=`ls -l $dir/root_uImage | awk '{print $5 }'`
  echo FW size $size
  fs_size=`expr $size - $kernel_block`
  echo fs size $fs_size

    real_md5=`md5sum $dir/$file | awk '{print $1}'`
    echo md5 of upgrade file: $real_md5 
    echo target md5 for upgrade: $target_md5 
}

update_kernel()
{
  if [ $size -ge $kernel_block ] ; then
      #gpio l 40 4000 0 1 0 4000
      echo 3 > /proc/sys/vm/drop_caches
      clear_history
      $burn_cmd -o 0 -l $kernel_block write $dir/$file Kernel  
      echo 3 > /proc/sys/vm/drop_caches
  fi
}


update_rootfs()
{
  if [ $fs_size -gt 0 ] ; then
      #gpio l 40 4000 0 1 0 4000
      echo 3 > /proc/sys/vm/drop_caches
      $burn_cmd -o $kernel_block -l $fs_size write $dir/$file RootFS  
      post_pro_file

      echo 3 > /proc/sys/vm/drop_caches
      # it can't use the mv for tf card :buggy 
      sync
      echo done  
      upgrade_result 1
      #gpio l 40 0 4000 1 0 4000
      if [ x$iot_upgrade = x"byiot" ]
      then
        sleep 6
      else
         sleep 3
       fi
      reboot -f 
      exit 0
  fi

  upgrade_result 2
}

update_rwfile()
{
    rm -rf /mnt/*
    cp -dprf rw_dir /mnt/creality
    sync
    rm_update_pack
    upgrade_result 1
    rm -fr rw_dir
    post_pro_file
    sync 
}

apply_update()
{
	/home/cmp_ver.sh restart
	
	#others..
	
}

use_rw_()
{

   if [ ! -f fast_update ]
   then
     return 0   
   fi

   if [ ! x$iot_upgrade = x"byiot" ]
   then
     return 0   
   fi

	if cat /proc/mtd |grep APP
	then	 
	  return 1
	fi
	return 0
}

rm_update_pack()
{
	rm -f $dir/cxsw_update.tar.bz2
}

use_rw_
if [ $? -eq 0 ]
then
  echo update fw
  rm_update_pack
  [ -f $dir/creality/batch-update ] && 
  {
    if diff $dir/ver /home/box_version
    then
      echo no need to upgrade
      exit 0
    fi  
  }
  update_boot
  if [ -f $dir/$file ] ;then  
    cal_size
    if [ x$target_md5 == x$real_md5 ] ; then
      update_kernel
      update_rootfs
    fi
  fi  
else
  echo update rw files
  if [ -d rw_dir ]
  then
    update_rwfile
    apply_update
     echo 3 > /proc/sys/vm/drop_caches
  fi
fi